using System;
using System.Windows.Forms;
using System.Collections;
using System.Xml;

using com.ibm.gre;

namespace SampleApp
{
	/// <summary>
	/// Summary description for CommonTasks.
	/// </summary>
	public class CommonTasks
	{
		public CommonTasks()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		//This method displays all the targets for selected nFlPlnCmpntID in ListView
		public static void DisplayTargetList(ClientSession session, ListView lv, int nFlPlnViewID, long nFlPlnCmpntID)
		{
			string xml = session.getNavigationController().getTargetList(nFlPlnViewID,nFlPlnCmpntID,0,"",0,0);
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
					
			lv.Items.Clear();
			lv.Columns.Clear();

			lv.Columns.Add(new ColumnHeader());
			lv.Columns[0].Text = "Component";
			lv.Columns[0].Width = 200;

			lv.Columns.Add(new ColumnHeader());
			lv.Columns[1].Text = "Name";
			lv.Columns[1].Width = 200;

			lv.Columns.Add(new ColumnHeader());
			lv.Columns[2].Text = "Title";
			lv.Columns[2].Width = 500;

			for(int i = 0; i < itemNodes.Count; i++)
			{
				IRMComponent irm = new IRMComponent(itemNodes[i]);
				ListViewItem lvItem = lv.Items.Add(new ListViewItem(
					new string[]{irm.FlPlnCmpntDefNm,irm.FlPlnCmpntNm,irm.FlPlnCmpntTtl}, 3 ));
				lvItem.Tag = irm;
			}
		}

		public static void viewSeletedItem(Control parent, ClientSession session, IRMComponent irm)
		{
			string xml = session.getTemplateController().getFilePlanComponent(irm.FlPlnCmpntID,0,(int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);
			IRMFields fields = new IRMFields(xml);
			fields.hideSecurityFields(); //This is a convenience method to hide security fields if they are not being used
			fields.inactivateSecurityFields(); //This is a convenience method to make sure security fields are not added
			ComponentForm frm = new ComponentForm(session,irm,parent,fields,ComponentForm.VIEW);
			frm.ShowDialog(parent);
		}

		public static void DisplayResultList(ListView lv, IRMResultList rl, int nFlPlnViewID)
		{
			lv.Items.Clear();
			lv.Columns.Clear();
			int colCount = rl.ColumnCount;
			float totalWidth = lv.Width;
			for(int i = 0; i < colCount; i++)
			{
				float percentWidth = rl.getPercentageColumnWidth(i);
				int width = (int)(totalWidth * (percentWidth/100));
				lv.Columns.Add(rl.getFlPlnCmpntAttrDefCapt(i),width,HorizontalAlignment.Left);
			}
			int itemCount = rl.ItemCount;
			for(int i = 0; i < itemCount; i++)
			{
				long nCurrentFlPlnCmpntID;
				string strFlPlnCmpntNm = "";
				string strFlPlnCmpntTtl = "";
				int nFlPlnCmpntDefID; 
				string strFlPlnCmpntDefNm = "";

				ListViewItem item = null;
				for(int j = 0; j < colCount; j++)
				{
					string strFlPlnCmpntAttrDefColNm = rl.getFlPlnCmpntAttrDefColNm(j);
					//string val = rl.getItemByFlPlnCmpntAttrDefColNm(i,strFlPlnCmpntAttrDefColNm);
					string val = rl.getItemByRowColumnIndex(i,j);
					if(j == 0)
					{
						int image = 0;
						item = new ListViewItem(val,image);
					}
					else
					{
						if(item != null)
						{
							item.SubItems.Add(val);
						}
					}
					//This Optional code is used only to add information to the listview tag 
					if(strFlPlnCmpntAttrDefColNm == "TS_FlPlnCmpntDef.FlPlnCmpntDefNm")
					{
						strFlPlnCmpntDefNm = val;
					}
					if(strFlPlnCmpntAttrDefColNm == "TS_FlPlnCmpnt.FlPlnCmpntNm")
					{
						strFlPlnCmpntNm = val;
					}
				}
				lv.Items.Add(item);
				nFlPlnCmpntDefID = Convert.ToInt32(rl.getItemByFlPlnCmpntAttrDefColNm(i,"ObjectType"));
				nCurrentFlPlnCmpntID = Convert.ToInt64(rl.getItemByFlPlnCmpntAttrDefColNm(i,"PrimaryKey"));
				IRMComponent currentItem = new IRMComponent(nCurrentFlPlnCmpntID,strFlPlnCmpntNm,strFlPlnCmpntTtl,nFlPlnCmpntDefID,strFlPlnCmpntDefNm,nFlPlnViewID);
				item.Tag = currentItem;
			}
		}

		public static string makeConflictName(string val)
		{
			int conflictType = Convert.ToInt32(val);
			switch(conflictType)
			{
				case (int)IRMConstants.ConflictTypes.ACTION_CONFLICT:
					return "ACTION_CONFLICT";
				case (int)IRMConstants.ConflictTypes.ELIGIBLE_DATE_CONFLICT:
					return "ELIGIBLE_DATE_CONFLICT";
				case (int)IRMConstants.ConflictTypes.MORE_THEN_ONE_CONFLICT:
					return "MORE_THEN_ONE_CONFLICT";
				default:
					return "";
			}
		}

		public static void DisplayLifeCycleResultList(ListView lv, IRMLifeCycleResultList rl, int nFlPlnViewID, bool rootsOnly)
		{
			lv.Items.Clear();
			lv.Columns.Clear();
			int colCount = rl.ColumnCount;
			float totalWidth = lv.Width;
			for(int i = 0; i < colCount; i++)
			{
				float percentWidth = rl.getPercentageColumnWidth(i);
				int width = (int)(totalWidth * (percentWidth/100));
				lv.Columns.Add(rl.getFlPlnCmpntAttrDefCapt(i),width,HorizontalAlignment.Left);
			}
			int itemCount = rl.ItemCount;
			for(int i = 0; i < itemCount; i++)
			{
				bool show = true;
				if(rootsOnly)
				{
					show = rl.isRoot(i);
				}
				if(show)
				{
					long nCurrentFlPlnCmpntID;
					string strFlPlnCmpntNm = "";
					string strFlPlnCmpntTtl = "";
					int nFlPlnCmpntDefID; 
					string strFlPlnCmpntDefNm = "";

					ListViewItem item = null;
					for(int j = 0; j < colCount; j++)
					{
						string strFlPlnCmpntAttrDefColNm = rl.getFlPlnCmpntAttrDefColNm(j);
						string val = rl.getItemByFlPlnCmpntAttrDefColNm(i,strFlPlnCmpntAttrDefColNm);
						//Add special code for the Conflict field
						if(strFlPlnCmpntAttrDefColNm == "Conflict")
						{
							val = makeConflictName(val);
						}
						if(j == 0)
						{
							int image = 3;
							if(rl.hasConflict(i))
							{
								image = 0;
							}
							item = new ListViewItem(val,image);
						}
						else
						{
							if(item != null)
							{
								item.SubItems.Add(val);
							}
						}
						//This Optional code is used only to add information to the listview tag 
						if(strFlPlnCmpntAttrDefColNm == "TS_FlPlnCmpntDef.FlPlnCmpntDefNm")
						{
							strFlPlnCmpntDefNm = val;
						}
						if(strFlPlnCmpntAttrDefColNm == "TS_FlPlnCmpnt.FlPlnCmpntNm")
						{
							strFlPlnCmpntNm = val;
						}
					}
					lv.Items.Add(item);
					nFlPlnCmpntDefID = Convert.ToInt32(rl.getItemByFlPlnCmpntAttrDefColNm(i,"ObjectType"));
					nCurrentFlPlnCmpntID = Convert.ToInt64(rl.getItemByFlPlnCmpntAttrDefColNm(i,"PrimaryKey"));
					IRMComponent currentItem = new IRMComponent(nCurrentFlPlnCmpntID,strFlPlnCmpntNm,strFlPlnCmpntTtl,nFlPlnCmpntDefID,strFlPlnCmpntDefNm,nFlPlnViewID);
					item.Tag = currentItem;
				}
			}
		}

		public static void DeleteListItems(Control parent, ClientSession session, Util util,  ListView lv)
		{
			int count = lv.SelectedItems.Count;
			if(count > 0)
			{
				string msg = "Remove Selected Items";
				IEnumerator items = lv.SelectedItems.GetEnumerator();
				
				if(count > 1)
				{
					if(MessageBox.Show(parent,"Delete Selected Components?","Component Operations",MessageBoxButtons.YesNo) == DialogResult.No)
					{
						return;
					}
				}

				IRMComponent irm;
				while(items.MoveNext())
				{
					ListViewItem lvi = (ListViewItem)items.Current;
					irm = (IRMComponent)lvi.Tag;
					if(count == 1)
					{
						if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
						{
							msg = "Delete File Plan Component: " + irm.FlPlnCmpntNm;
						}
						if(irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
						{
							msg = "Delete File Plan View: " + irm.FlPlnViewNm;
						}
						if(MessageBox.Show(parent,msg,"Component Operations",MessageBoxButtons.YesNo) == DialogResult.No)
						{
							return;
						}
					}
					try
					{
						if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
						{
							session.getFilePlanComponentController().deleteFilePlanComponent(irm.FlPlnCmpntID,1);
						}
						if(irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
						{
							session.getFilePlanDesignController().deleteFilePlanView(irm.FlPlnViewID);
						}
					}
					catch(System.Net.WebException we)
					{
						util.logWebException(we);
						return;
					}
					catch(System.Web.Services.Protocols.SoapException s)
					{
						util.handleSoapException(s,false); //false indicates we will not check for time out here
						util.showSoapException(s);
						return;
					}
					lvi.Remove();
				}
			}			
		}

		public static string getEngineConfigValue(ClientSession session, Util util, string key)
		{
			string val = "";
			try
			{
				val = session.getConfigurationController().getEngineConfigValue(key,0);
			}
			catch(System.Net.WebException we)
			{
				util.logWebException(we);
			}
			catch(System.Web.Services.Protocols.SoapException s)
			{
				util.handleSoapException(s,false); //false indicates we will not check for time out here
			}
			return val;
		}


		public static void DrawFilePlan(ClientSession session, TreeView tv)
		{

			TreeNode curTreeNode = null;

			//Create a new element
			TreeNode viewTreeNode = new TreeNode("Views");
			
			viewTreeNode.ImageIndex = 1;
			viewTreeNode.SelectedImageIndex = 2;

			IRMComponent irmViews = new IRMComponent();
			tv.Nodes.Add(viewTreeNode);
			viewTreeNode.Tag = irmViews;

			string xml = session.getFilePlanDesignController().getFilePlanViewList(true);
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
			int count = itemNodes.Count;
			if(count > 0)
			{
				irmViews.HasTgt = true;
			}
			for(int i = 0; i < count; i++)
			{
				int nFlPlnViewID = System.Convert.ToInt32(itemNodes.Item(i).Attributes.GetNamedItem("FlPlnViewID").Value);
				string strFlPlnViewNm = itemNodes.Item(i).Attributes.GetNamedItem("FlPlnViewNm").Value;

				IRMComponent irm = new IRMComponent(nFlPlnViewID,strFlPlnViewNm);

				//Create a new element
				curTreeNode = new TreeNode(strFlPlnViewNm);

				curTreeNode.ImageIndex = 1;
				curTreeNode.SelectedImageIndex = 2;

				//Add the component information to the Tag
				curTreeNode.Tag = irm;

				viewTreeNode.Nodes.Add(curTreeNode);

				addMarker(session,curTreeNode,nFlPlnViewID,0);
			}

		}

		private static void addMarker(ClientSession session, TreeNode parent, int nFlPlnViewID,long nFlPlnCmpntID)
		{
			string xml = session.getNavigationController().getTargetList(nFlPlnViewID,nFlPlnCmpntID,0,"",0,0);
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
			int count = itemNodes.Count;
			if(count > 0)
			{
				parent.Nodes.Add(new TreeNode("MARKER"));
			}
		}
		
		public static void addTargetList(ClientSession session, TreeNode parent, int nFlPlnViewID,long nFlPlnCmpntID)
		{
			string xml = session.getNavigationController().getTargetList(nFlPlnViewID,nFlPlnCmpntID,0,"",0,0);
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
			int count = itemNodes.Count;
			if(count > 0)
			{
				if(parent.Nodes.Count == 1)
				{
					//Check if this is a Marker
					if(parent.FirstNode.Tag == null)
					{
						parent.FirstNode.Remove();
					}
				}
				for(int i = 0; i < count; i++)
				{
					IRMComponent irm = new IRMComponent(itemNodes.Item(i));
					//Create a new element
					TreeNode curTreeNode = new TreeNode(irm.FlPlnCmpntNm);
					//Add the component information to the Tag
					curTreeNode.Tag = irm;
					//TODO image assignment under construction
					curTreeNode.ImageIndex = 3;
					curTreeNode.SelectedImageIndex = 4;
					parent.Nodes.Add(curTreeNode);
					if(irm.HasTgt)
					{
						curTreeNode.Nodes.Add(new TreeNode("MARKER"));
					}
				}
			}
		}

		/// <summary>
		/// This method will do a refresh on all existing View Nodes.
		/// </summary>
		/// <param name="parent"></param>
		/// <param name="deep"></param>
		public static void RefreshViews(ClientSession session, TreeNode parent, bool deep)
		{
			TreeNodeCollection viewNodes = parent.Nodes;
			string xml = session.getFilePlanDesignController().getFilePlanViewList(true);
			XmlDocument doc = new XmlDocument();

			doc.LoadXml(xml);
			XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
			int count = itemNodes.Count;

			//First remove all unused nodes
			NodeHelper hlpr = new NodeHelper(itemNodes, "FlPlnViewID");
			for(int i = viewNodes.Count - 1; i != -1; i--)
			{
				IRMComponent irm = (IRMComponent)viewNodes[i].Tag;
				if(!hlpr.findId(irm.FlPlnViewID))
				{
					viewNodes[i].Remove();
				}
			}
			if(count > 0)
			{
				for(int i = 0; i < count; i++)
				{
					int nFlPlnViewID = System.Convert.ToInt32(itemNodes.Item(i).Attributes.GetNamedItem("FlPlnViewID").Value);
					string strFlPlnViewNm = itemNodes.Item(i).Attributes.GetNamedItem("FlPlnViewNm").Value;
					IRMComponent irmNew = new IRMComponent(nFlPlnViewID,strFlPlnViewNm);

					//This is the case where new elements were added by another client
					//and were not added when the tree was first constructed
					//The index we are at in the xml is larger than the last tree node index 
					if(viewNodes.Count - 1 < i)
					{
						//Create a new element
						TreeNode newTreeNode = new TreeNode(strFlPlnViewNm);
						newTreeNode.ImageIndex = 1;
						newTreeNode.SelectedImageIndex = 2;
						//Add the component information to the Tag
						newTreeNode.Tag = irmNew;
						viewNodes.Insert(i,newTreeNode);
						addMarker(session,newTreeNode,nFlPlnViewID,0);
					}
					else
					{
						IRMComponent irmCurrent = (IRMComponent)viewNodes[i].Tag;
						//Check if both items have the same ID
						if(irmNew.FlPlnViewID == irmCurrent.FlPlnViewID)
						{
							//update current info
							viewNodes[i].Text = irmNew.FlPlnViewNm;
							viewNodes[i].Tag = irmNew;
							
							if(deep) //Recusively refresh child nodes
							{
								Refresh(session,viewNodes[i], nFlPlnViewID,0, deep,false);
							}
						}
						else
						{
							//Create a new element
							TreeNode newTreeNode = new TreeNode(strFlPlnViewNm);
							newTreeNode.ImageIndex = 1;
							newTreeNode.SelectedImageIndex = 2;
							//Add the component information to the Tag
							newTreeNode.Tag = irmNew;
							viewNodes.Insert(i,newTreeNode);
							addMarker(session,newTreeNode,nFlPlnViewID,0);
						}
					}
				}
			}
			else
			{
				parent.Nodes.Clear();
			}
		}

		public static void Refresh(ClientSession session, TreeNode parent, int nFlPlnViewID,long nFlPlnCmpntID, bool deep, bool checkTag)
		{
			TreeNodeCollection nodes = parent.Nodes;
			if(nodes.Count == 1)
			{
				if(parent.FirstNode.Tag == null)
				{
					if(checkTag) //When items are being deleted we should remove the Marker for any empty lists
					{
						string tempXml = session.getNavigationController().getTargetList(nFlPlnViewID,nFlPlnCmpntID,0,"",0,0);
						XmlDocument tempDoc = new XmlDocument();
						tempDoc.LoadXml(tempXml);
						XmlNodeList tempNodes = tempDoc.GetElementsByTagName("Item");
						if(tempNodes.Count == 0)
						{
							parent.FirstNode.Remove();
						}
					}
					return;
				}
			}
			string xml = session.getNavigationController().getTargetList(nFlPlnViewID,nFlPlnCmpntID,0,"",0,0);
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
			int count = itemNodes.Count;
			
			//First remove all unused nodes
			NodeHelper hlpr = new NodeHelper(itemNodes, "FlPlnCmpntID");
			for(int i = nodes.Count - 1; i != -1; i--)
			{
				IRMComponent irm = (IRMComponent)nodes[i].Tag;
				if(!hlpr.findId(irm.FlPlnCmpntID))
				{
					nodes[i].Remove();
				}
			}
			//Update or insert new items
			if(count > 0)
			{
				for(int i = 0; i < count; i++)
				{
					IRMComponent irmNew = new IRMComponent(itemNodes.Item(i));

					//This is the case where new elements were added by another client
					//and were not added when the tree was first constructed
					if(nodes.Count - 1 < i)
					{
						//Create a new element
						TreeNode newTreeNode = new TreeNode(irmNew.FlPlnCmpntNm);
						//Add the component information to the Tag
						newTreeNode.Tag = irmNew;
						//TODO image assignment under construction
						newTreeNode.ImageIndex = 3;
						newTreeNode.SelectedImageIndex = 4;
						nodes.Insert(i,newTreeNode);
						if(irmNew.HasTgt)
						{
							newTreeNode.Nodes.Add(new TreeNode("MARKER"));
						}
					}
					else
					{
						IRMComponent irmCurrent = (IRMComponent)nodes[i].Tag;

						//Check if both items have the same ID
						if(irmNew.FlPlnCmpntID == irmCurrent.FlPlnCmpntID)
						{
							//update current info
							nodes[i].Text = irmNew.FlPlnCmpntNm;
							nodes[i].Tag = irmNew;
							if(deep) //Recusively refresh child nodes
							{
								Refresh(session,nodes[i], nFlPlnViewID,irmNew.FlPlnCmpntID, deep,checkTag);
							}
						}
						else
						{
							//Create a new element
							TreeNode newTreeNode = new TreeNode(irmNew.FlPlnCmpntNm);
							//Add the component information to the Tag
							newTreeNode.Tag = irmNew;
							//TODO image assignment under construction
							newTreeNode.ImageIndex = 3;
							newTreeNode.SelectedImageIndex = 4;
							nodes.Insert(i,newTreeNode);
							if(irmNew.HasTgt)
							{
								newTreeNode.Nodes.Add(new TreeNode("MARKER"));
							}
						}
					}
				}
			}
			else
			{
				parent.Nodes.Clear();
			}
		}
	}
}
